# FillDispensers Filter by SethBling
# Feel free to modify and reuse, but credit to SethBling would be nice.

from pymclevel import MCSchematic
from pymclevel import TileEntity
from pymclevel import TAG_Compound
from pymclevel import TAG_Short
from pymclevel import TAG_Byte
from pymclevel import TAG_String
from numpy import zeros
import random


inputs = (
	("Quantity", (127, -1, 127)),
	("Fill empty slots", False),
	("Fill item id", (0, 0, 2266)),
	("Fill item damage", 0),
)

def perform(level, box, options):
	qty = options["Quantity"]
	fill = options["Fill empty slots"]
	fillid = options["Fill item id"]
	filldmg = options["Fill item damage"]

	minx = int(box.minx/16)*16
	minz = int(box.minz/16)*16

	for x in xrange(minx, box.maxx, 16):
		for z in xrange(minz, box.maxz, 16):
			chunk = level.getChunk(x / 16, z / 16)

			for te in chunk.TileEntities:
				px = te["x"].value
				py = te["y"].value
				pz = te["z"].value

				if px < box.minx or px >= box.maxx:
					continue
				if py < box.miny or py >= box.maxy:
					continue
				if pz < box.minz or pz >= box.maxz:
					continue

				numSlots = 0
				if te["id"].value == "Trap":
					numSlots = 9
				if te["id"].value == "Dropper":
					numSlots = 9
				if te["id"].value == "Chest":
					numSlots = 27
				
				if numSlots != 0:
					slots = zeros(numSlots)
					for item in te["Items"]:
						item["Count"] = TAG_Byte(qty)
						slot = item["Slot"].value
						if slot >= 0 and slot < numSlots:
							slots[slot] = 1

					if fill:
						for i in range(numSlots):
							if slots[i] == 0:
								item = TAG_Compound()
								item["id"] = TAG_Short(fillid)
								item["Damage"] = TAG_Short(filldmg)
								item["Count"] = TAG_Byte(qty)
								item["Slot"] = TAG_Byte(i)
								te["Items"].append(item)

					chunk.dirty = True
					
