# Feel free to modify and use this filter however you wish. If you do,
# please give credit to SethBling.
# http://youtube.com/SethBling

from numpy import zeros
from pymclevel import TAG_Int

def perform(level, box, options):
	width = box.maxx - box.minx
	height = box.maxy - box.miny
	depth = box.maxz - box.minz
	blocks = width * height * depth
	labels = zeros(blocks).reshape(width, height, depth)
	groups = {}
	
	id = 0
	
	for x in xrange(box.minx, box.maxx):
		for y in xrange(box.miny, box.maxy):
			for z in xrange(box.minz, box.maxz):
				floodFill(labels, groups, level, box, x, y, z, id)
				id = id + 1
	
	for id in groups:
		flip(groups[id], level)

NaturalBlocks = [0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 21, 24, 31, 32, 49, 56, 73, 74, 78, 79, 81, 82, 83, 86, 106, 110, 111, 129, ]
Stairs = [53, 67, 108, 109, 114, 128, 134, 135, 136, 156]
Slabs = [44, 126]
				
def floodFill(labels, groups, level, box, x, y, z, id):
	if x < box.minx or x >= box.maxx:
		return
	if y < box.miny or y >= box.maxy:
		return
	if z < box.minz or z >= box.maxz:
		return
	
	lx = x - box.minx
	ly = y - box.miny
	lz = z - box.minz
	if labels[lx, ly, lz] != 0:
		return
	
	block = level.blockAt(x,y,z)
	if block in NaturalBlocks:
		return
	
	labels[lx, ly, lz] = id
	if id not in groups:
		groups[id] = []
	groups[id].append((x, y, z))
	
	floodFill(labels, groups, level, box, x-1, y, z, id)
	floodFill(labels, groups, level, box, x+1, y, z, id)
	floodFill(labels, groups, level, box, x, y-1, z, id)
	floodFill(labels, groups, level, box, x, y+1, z, id)
	floodFill(labels, groups, level, box, x, y, z-1, id)
	floodFill(labels, groups, level, box, x, y, z+1, id)

def flip(group, level):
	maxy = 0
	miny = 0
	first = True
	for (x, y, z) in group:
		if y > maxy or first:
			maxy = y
		if y < miny or first:
			miny = y
		first = False
	
	midy = (miny + maxy) / 2.0
	
	columnHeights = {}
	for (x, y, z) in group:
		if (x, z) not in columnHeights:
			columnHeights[(x, z)] = 0
		dy = abs(midy - y)
		if dy > columnHeights[(x, z)]:
			columnHeights[(x, z)] = dy
	
	for (x, z) in columnHeights:
		height = columnHeights[(x, z)]
		dy = height
		while dy > 0:
			y = int(midy + dy)
			swapy = int(midy - dy)
			tempBlock = level.blockAt(x, y, z)
			tempData = level.blockDataAt(x, y, z)
			tempTileEntity = level.tileEntityAt(x, y, z)
			swapTileEntity = level.tileEntityAt(x, swapy, z)
			
			block = level.blockAt(x, swapy, z)
			data = level.blockDataAt(x, swapy, z)
			if block in Stairs:
				data = data ^ 0x04
			if block in Slabs:
				data = data ^ 0x08
			level.setBlockAt(x, y, z, block)
			level.setBlockDataAt(x, y, z, data)
			if tempTileEntity != None:
				tempTileEntity["y"] = TAG_Int(swapy)
			
			if tempBlock in Stairs:
				tempData = tempData ^ 0x04
			if tempBlock in Slabs:
				tempData = tempData ^ 0x08
			level.setBlockAt(x, swapy, z, tempBlock)
			level.setBlockDataAt(x, swapy, z, tempData)
			if swapTileEntity != None:
				swapTileEntity["y"] = TAG_Int(y)
			
			dy = dy - 1
	
	for (x, z) in columnHeights:
		if columnHeights[(x, z)] - 0.5 != int(columnHeights[(x, z)]):
			block = level.blockAt(x, midy, z)
			data = level.blockDataAt(x, midy, z)
			if block in Stairs:
				level.setBlockDataAt(x, midy, z, data^0x04)
			if block in Slabs:
				level.setBlockDataAt(x, midy, z, data^0x08)