# Feel free to modify and use this filter however you wish. If you do,
# please give credit to SethBling.
# http://youtube.com/SethBling

import random

def perform(level, box, options):
	width = box.maxx - box.minx
	depth = box.maxz - box.minz
	height = box.maxy - box.miny
	if width % 2 == 0:
		box.maxx = box.maxx - 1
	if depth % 2 == 0:
		box.maxz = box.maxz - 1
	
	# Set up the initial skeleton, clearing only the intersections
	for x in xrange(box.minx, box.maxx):
		for z in xrange(box.minz, box.maxz):
			for y in xrange(box.miny, box.maxy):
				if (x - box.minx) % 2 == 1 and (z - box.minz) % 2 == 1:
					level.setBlockAt(x, y, z, 0)
					level.setBlockDataAt(x, y, z, 0)
				else:
					level.setBlockAt(x, y, z, 18)
					level.setBlockDataAt(x, y, z, 4)

	# Label every intersection so that we can tell when they're all connected
	groups = {}
	group = 0
	for x in xrange(box.minx+1, box.maxx-1, 2):
		for z in xrange(box.minz+1, box.maxz-1, 2):
			groups[(x, z)] = group
			group = group + 1

	# The goal of this algorithm is to connect all intersections via randomized
	# pathways. In each iteration, it picks a random intersection, and tries to
	# connect it to another intersection that it wasn't already connected to.
	# Once all intersections are connected via some pathway, the algorithm is
	# done.
	while True:
		# Pick random intersection
		x = box.minx + random.randint(0, (width-1)/2-1) * 2 + 1
		z = box.minz + random.randint(0, (depth-1)/2-1) * 2 + 1
		
		# Pick random direction
		dir = random.randint(0, 3)
		if dir == 0:
			(dx, dz) = (-1, 0)
		if dir == 1:
			(dx, dz) = (1, 0)
		if dir == 2:
			(dx, dz) = (0, -1)
		if dir == 3:
			(dx, dz) = (0, 1)

		mx = x + dx
		mz = z + dz
				
		if level.blockAt(mx, box.miny, mz) == 0:
			# This path has already been cleared
			continue
			
		thisid = groups[(x, z)]
		otherSpot = (x + 2*dx, z + 2*dz)
		if otherSpot not in groups:
			# We've hit the edge of the maze
			continue
		oldid = groups[otherSpot]
		
		if oldid == thisid:
			# These two intersections are already connected
			continue
			
		# Update the connecting group to match this one, since they're now connected
		for spot in groups:
			if groups[spot] == oldid:
				groups[spot] = thisid
		
		# Clear the path
		for y in xrange(box.miny, box.maxy):
			level.setBlockAt(mx, y, mz, 0)
			level.setBlockDataAt(mx, y, mz, 0)
		
		# Check if all intersections belong to the same group (aka are connected)
		done = True
		for spot in groups:
			if groups[spot] != thisid:
				done = False
				break
		
		if done:
			break