# coding unicode-escape
# Feel free to modify and use this filter however you wish. If you do,
# please give credit to SethBling.
# http://youtube.com/SethBling

from pymclevel import TAG_List
from pymclevel import TAG_Byte
from pymclevel import TAG_Int
from pymclevel import TAG_Compound
from pymclevel import TAG_Short
from pymclevel import TAG_Double
from pymclevel import TAG_String

displayName = "Place Sign Text"

Colors = {
	"Black": "0",
	"Dark Blue": "1",
	"Dark Green": "2",
	"Dark Aqua": "3",
	"Dark Red": "4",
	"Purple": "5",
	"Gold": "6",
	"Gray": "7",
	"Dark Gray": "8",
	"Indigo": "9",
	"Bright Green": "a",
	"Aqua": "b",
	"Red": "c",
	"Pink": "d",
	"Yellow": "e",
	"White": "f",
	}

inputs = (
	("Text", "string"),
	("Text Flow", ("Top to Bottom", "Left to Right")),
	("Use Color", False),
	("Color", tuple(Colors.keys())),
)

formatCode = unichr(167)

def perform(level, box, options):
	text = options["Text"]
	color = Colors[options["Color"]]
	ltr = (options["Text Flow"] == "Left to Right")
	if ltr:
		xzs = 1
		ys = 1000
	else:
		xzs = 1000
		ys = 1
	
	formatCount = 0
	if options["Use Color"]:
		formatCount = formatCount + 1

	lineLength = 15 - 2*formatCount

	signs = []
	dir = 2
	
	for (chunk, slices, point) in level.getChunkSlices(box):
		for t in chunk.TileEntities:
			x = t["x"].value
			y = t["y"].value
			z = t["z"].value
			
			if x >= box.minx and x < box.maxx and y >= box.miny and y < box.maxy and z >= box.minz and z < box.maxz and t["id"].value == "Sign":
				signs.append(t)
				dir = level.blockDataAt(x, y, z)
	
	sortedSigns = []
	while len(signs) > 0:
		first = True
		for sign in signs:
			if dir == 2:
				score = -sign["x"].value*xzs - sign["y"].value*ys
			if dir == 3:
				score = sign["x"].value*xzs - sign["y"].value*ys
			if dir == 4:
				score = sign["z"].value*xzs - sign["y"].value*ys
			if dir == 5:
				score = -sign["z"].value*xzs - sign["y"].value*ys
				
			if first or score < minScore:
				minScore = score
				minSign = sign
				first = False
		
		signs.remove(minSign)
		sortedSigns.append(minSign)
	
	for sign in sortedSigns:
		print str(sign["x"].value) + ", " + str(sign["y"].value) + ", " + str(sign["z"].value)
		for lineNum in range(1, 5):
			line = ""
			if len(text) <= lineLength:
				line = text
				text = ""
			else:
				for i in range(lineLength, -1, -1):
					if text[i] == ' ':
						line = text[:i]
						text = text[i+1:]
						break
				if line == "":
					line = text[:lineLength]
					text = text[lineLength:]
			
			if options["Use Color"]:
				line = formatCode + color + line
				
			sign["Text" + str(lineNum)] = TAG_String(line)
	
	level.markDirtyBox(box)
